// Interrupt handling TIMER 0 FOR REAL TIME CLOCK
// Compiler C18 V3.40, MPLAB V8.63, 18F2550 BOLT SYSTEM
// REAL TIME CLOCK, TIME IN LCD
// Handles, seconds, minutes and hours. You can add the
// OPTION DAY, MONTH AND YEAR WITHOUT PROBLEM.
// The format is 24 hours.
// When the program starts, it asks the user to enter
// TIME THROUGH THE KEYPAD. PROMT
// "SET" is displayed on the LCD. The user must type TIME
// WITH FORMAT six consecutive digits example: 102345 (type "F")
// which means 10:23:45

#include <p18cxxx.h>
#include <delays.h>
#include <timers.h>
#include <stdio.h>
#include <stdlib.h>
#include "18F2550BOLT.h"
#define CLOCK_TIMER0 12000000;	//es la frecuencia (hertz) con la que se modifica el registro de 16 bits del TIMER0

void timer_isr(void);
void LCD_CEROS(void);
void DISPLAY_LCD(void);
void SET_TIME(void);

extern void _startup( void ); // See c018i.c in your C18 compiler dir 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code _HIGH_INTERRUPT_VECTOR = 0x000808
void _high_ISR (void)
{
	_asm goto timer_isr _endasm
}

#pragma code
int long Ticker; 
int Segundos=0; 
int segundo_previo=0;
int Year=0,Mes=0,Dia=0,Hora=0,Minuto=0; 
#define BISIESTO(Year) (Year%4 == 0);

#pragma interrupt timer_isr

void timer_isr(void)
{
Ticker=Ticker-65536;                    // Decrementa ticker  
  if (Ticker<65536)                      // si ya transcurri un segundo 
  {
	Ticker=Ticker+CLOCK_TIMER0;      	//   Incrementa ticker 
    Segundos++;                           //   Incrementa numero de segundos 
  } 
				if(Segundos == 60) 
				{
				Minuto++; Segundos=0; 
    				if(Minuto == 60) 
					{
					Hora++; Minuto=0; 
      					if(Hora == 24) 
						{
						Dia++; Hora=0; 
  						} 
					}		
				}  
INTCONbits.TMR0IF = 0 ;				//habilita de nuevo interrupcin
}

void main() 
{
	init_bolt();
	PORTB=0;
	SET_TIME();
	Ticker=CLOCK_TIMER0;
	OpenTimer0 (TIMER_INT_ON & T0_SOURCE_INT & T0_16BIT);			//INICIALIZA PARAMETROS DEL TIMER 0
	INTCONbits.GIE = 1; 											//enable global interrupts
	LCD_CEROS();
  		while(1) 
  			{ 
			if (Segundos != segundo_previo) 
    			{
				segundo_previo=Segundos; 
      			DISPLAY_LCD();
    			}  
  			} 
} 

void LCD_CEROS(void)
{
 char HORA[]="TIME:";
 char CEROS[10]="00:00:00";		//HORA INICIAL
 InitLCD();
 ClearScreen();
 PrintString(HORA);
 GotoXY(1,0);
 PrintString(CEROS);
} 

void DISPLAY_LCD(void)
{
 char wob[10];
 GotoXY(1,0);
 sprintf(wob,"%02u",Hora);
 PrintString(wob); 
 GotoXY(1,3);
 sprintf(wob,"%02u",Minuto);
 PrintString(wob); 
 GotoXY(1,6);
 sprintf(wob,"%02u",Segundos);
 PrintString(wob); 
}

void SET_TIME(void)
{
char buf[3];
char SET[]="SET>";
InitLCD();
ClearScreen();
PrintString(SET);
GotoXY(1,0);
SetBot();				//INGRESA LA HORA (6 DIGITOS CONSECUTIVOS) Y "F" AL FINAL
GotoXY(1,0);
WrtBot();
buf[0]=bot[0];
buf[1]=bot[1];
buf[2]=0;
Hora=atoi(buf);
buf[0]=bot[2];
buf[1]=bot[3];
buf[2]=0;
Minuto=atoi(buf);
buf[0]=bot[4];
buf[1]=bot[5];
buf[2]=0;
Segundos=atoi(buf);
}